//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013-2015 Nicolas Froment, Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//  Copyright (C) 2015 Joachim Schmitz
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//
//  FNcolournotes is a modified version of the colornotes plugin created by Drake Music
//  Scotland. It colours the notes according to the Figurenotes colour format.
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      version:  "1.0"
      description: "This plugin colours notes according to the Figurenotes colour format"
      menuPath: "Plugins.Notes.Figurenotes Colour Notes"

      property variant colors : [
        /* Fbb */ "#00a6d6" , /* Cbb */ "#ff0000" , /* Gbb */ "#0c0c0c" , /* Dbb */ "#aa753f" ,
        /* Abb */ "#f9d514" , /* Ebb */ "#a7a398" , /* Bbb */ "#1eb53a",
        /* Fb */ "#00a6d6" , /* Cb */ "#ff0000" , /* Gb */ "#0c0c0c" , /* Db */ "#aa753f" ,
        /* Ab */ "#f9d514" , /* Eb */ "#a7a398" , /* Bb */ "#1eb53a" ,
        /* F */ "#00a6d6" , /* C */ "#ff0000" , /* G */ "#0c0c0c" , /* D */ "#aa753f" ,
        /* A */ "#f9d514" , /* E */ "#a7a398" , /* B */ "#1eb53a" ,
        /* F# */ "#00a6d6" , /* C# */ "#ff0000" , /* G# */ "#0c0c0c" , /* D# */ "#aa753f" ,
        /* A# */ "#f9d514" , /* E# */ "#a7a398" , /* B# */ "#1eb53a" ,
        /* F## */ "#00a6d6" , /* C## */ "#ff0000" , /* G## */ "#0c0c0c" , /* D## */ "#aa753f" ,
        /* A## */ "#f9d514" , /* E## */ "#a7a398" , /* B## */ "#1eb53a"
            ]

      property variant black : "black"

      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score

      function applyToNotesInSelection(func) {
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  if (cursor.tick == 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind(2) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  for (var voice = 0; voice < 4; voice++) {
                        cursor.rewind(1); // sets voice to 0
                        cursor.voice = voice; //voice has to be set after goTo
                        cursor.staffIdx = staff;

                        if (fullScore)
                              cursor.rewind(0) // if no selection, beginning of score

                        while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                              if (cursor.element && cursor.element.type == Element.CHORD) {
                                    var graceChords = cursor.element.graceNotes;
                                    for (var i = 0; i < graceChords.length; i++) {
                                          // iterate through all grace chords
                                          var notes = graceChords[i].notes;
                                          for (var j = 0; j < notes.length; j++)
                                                func(notes[j]);
                                    }
                                    var notes = cursor.element.notes;
                                    for (var i = 0; i < notes.length; i++) {
                                          var note = notes[i];
                                          func(note);
                                    }
                              }
                              cursor.next();
                        }
                  }
            }
      }

      function colorNote(note) {
            if (note.color == black)
                  note.color = colors[(note.tpc + 1) % 35];
            else
                  note.color = black;

            if (note.accidental) {
                  note.accidental.color = black;
                  }

            for (var i = 0; i < note.dots.length; i++) {
                  if (note.dots[i]) {
                        if (note.dots[i].color == black)
                              note.dots[i].color = colors[(note.tpc + 1) % 35];
                        else
                              note.dots[i].color = black;
                        }
                  }
         }

      onRun: {
            console.log("hello colornotes");

            if (typeof curScore === 'undefined')
                  Qt.quit();

            applyToNotesInSelection(colorNote)

            Qt.quit();
         }
}
